<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Command\Logout;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Command\BaseCommand;
use Org\Mxchange\CoreFramework\Command\Commandable;
use Org\Mxchange\CoreFramework\Controller\Controller;
use Org\Mxchange\CoreFramework\Factory\Object\ObjectFactory;
use Org\Mxchange\CoreFramework\Helper\Application\ApplicationHelper;
use Org\Mxchange\CoreFramework\Registry\GenericRegistry;
use Org\Mxchange\CoreFramework\Request\Requestable;
use Org\Mxchange\CoreFramework\Resolver\Command\CommandResolver;
use Org\Mxchange\CoreFramework\Response\Responseable;

/**
 * A command for the login failed page
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2023 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class HtmlLogoutDoneCommand extends BaseCommand implements Commandable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$resolverInstance	An instance of a command resolver class
	 * @return	$commandInstance	An instance a prepared command class
	 */
	public static final function createHtmlLogoutDoneCommand (CommandResolver $resolverInstance) {
		// Get new instance
		$commandInstance = new HtmlLogoutDoneCommand();

		// Set the application instance
		$commandInstance->setResolverInstance($resolverInstance);

		// Return the prepared instance
		return $commandInstance;
	}

	/**
	 * Executes the given command with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get the application instance
		$applicationInstance = ApplicationHelper::getSelfInstance();

		// Assign application data
		$this->getTemplateInstance()->assignApplicationData();

		// Load the master template
		$masterTemplate = $applicationInstance->buildMasterTemplateName();

		// Load header template
		$this->getTemplateInstance()->loadCodeTemplate('header');

		// Compile and assign it with a variable
		$this->getTemplateInstance()->compileTemplate();
		$this->getTemplateInstance()->assignTemplateWithVariable('header', 'header');

		// Load footer template
		$this->getTemplateInstance()->loadCodeTemplate('footer');

		// Compile and assign it with a variable
		$this->getTemplateInstance()->compileTemplate();
		$this->getTemplateInstance()->assignTemplateWithVariable('footer', 'footer');

		// Load the logout_done template
		$this->getTemplateInstance()->loadCodeTemplate('logout_done');

		// Assign the logout_done template with the master template as a content ... ;)
		$this->getTemplateInstance()->assignTemplateWithVariable('logout_done', 'main_content');

		// Load the master template
		$this->getTemplateInstance()->loadCodeTemplate($masterTemplate);

		// Set title
		$this->getTemplateInstance()->assignVariable('title', FrameworkBootstrap::getLanguageInstance()->getMessage('logout_done_title'));

		// Assign base URL
		$this->getTemplateInstance()->assignConfigVariable('base_url');

		// Construct the menu in every command. We could do this in BaseCommand class. But this means
		// *every* command has a navigation system and that is want we don't want.
		$menuInstance = ObjectFactory::createObjectByConfiguredName('logout_menu_class');

		// Render the menu
		$menuInstance->renderMenu();

		// Transfer it to the template engine instance
		$menuInstance->transferContentToTemplateEngine($this->getTemplateInstance());

		/*
		 * ... and all variables. This should be merged together in a pattern
		 * to make things easier. A cache mechanism should be added between
		 * these two calls to cache compiled templates.
		 */
		$this->getTemplateInstance()->compileVariables();

		// Get the content back from the template engine and put it in response class
		$this->getTemplateInstance()->transferToResponse($responseInstance);
	}

	/**
	 * Adds extra filters to the given controller instance
	 *
	 * @param	$controllerInstance		A controller instance
	 * @param	$requestInstance		An instance of a class with an Requestable interface
	 * @return	void
	 */
	public function addExtraFilters (Controller $controllerInstance, Requestable $requestInstance) {
		// Empty for now
	}

}
