<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Mailer;

// Import framework stuff
use Org\Mxchange\CoreFramework\Generic\FrameworkInterface;
use Org\Mxchange\CoreFramework\Manager\Login\ManageableMember;

/**
 * An interface for deliverable mail aka. mailer classes
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2023 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface DeliverableMail extends FrameworkInterface {
	/**
	 * Adds a user class to the recipient list for current template
	 *
	 * @param	$userInstance	An instance of a user class
	 * @return	void
	 */
	function addRecipientByUserInstance (ManageableMember $userInstance);

	/**
	 * Use subject line provided by the (XML) template otherwise a subject line must be set
	 *
	 * @return	void
	 */
	function useSubjectFromTemplate ();

	/**
	 * Deliver email to the recipient(s)
	 *
	 * @return	void
	 */
	function deliverEmail ();

	/**
	 * Send notification to the admin
	 *
	 * @return	void
	 */
	function sendAdminNotification ();

}
